@@ -28,7 +28,12 @@
 #include <gio/giotypes.h>
 
 #include <gio/gappinfo.h>
+#include <gio/gaction.h>
+#include <gio/gsimpleaction.h>
+#include <gio/gactiongroup.h>
+#include <gio/gsimpleactiongroup.h>
 #include <gio/gapplication.h>
+#include <gio/gapplicationcommandline.h>
 #include <gio/gasyncinitable.h>
 #include <gio/gasyncresult.h>
 #include <gio/gbufferedinputstream.h>
@@ -52,7 +57,6 @@
 #include <gio/gdbusnameowning.h>
 #include <gio/gdbusnamewatching.h>
 #include <gio/gdbusproxy.h>
-#include <gio/gdbusproxywatching.h>
 #include <gio/gdbusserver.h>
 #include <gio/gdbusutils.h>
 #include <gio/gdrive.h>
@@ -89,7 +93,12 @@
 #include <gio/gnetworkaddress.h>
 #include <gio/gnetworkservice.h>
 #include <gio/goutputstream.h>
+#include <gio/gperiodic.h>
 #include <gio/gpermission.h>
+#include <gio/gproxy.h>
+#include <gio/gproxyaddress.h>
+#include <gio/gproxyaddressenumerator.h>
+#include <gio/gproxyresolver.h>
 #include <gio/gresolver.h>
 #include <gio/gseekable.h>
 #include <gio/gsettings.h>
