@@ -34,6 +34,7 @@
 
 #ifdef G_OS_UNIX
 #include "gunixresolver.h"
+#include <sys/stat.h>
 #endif
 #ifdef G_OS_WIN32
 #include "gwin32resolver.h"
@@ -41,7 +42,6 @@
 
 #include <stdlib.h>
 
-#include "gioalias.h"
 
 /**
  * SECTION:gresolver
@@ -58,6 +58,21 @@
  * making it easy to connect to a remote host/service.
  */
 
+enum {
+  RELOAD,
+  LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL] = { 0 };
+
+struct _GResolverPrivate {
+#ifdef G_OS_UNIX
+  time_t resolv_conf_timestamp;
+#else
+  int dummy;
+#endif
+};
+
 /**
  * GResolver:
  *
@@ -71,6 +86,8 @@ g_resolver_class_init (GResolverClass *resolver_class)
 {
   volatile GType type;
 
+  g_type_class_add_private (resolver_class, sizeof (GResolverPrivate));
+
   /* Make sure _g_networking_init() has been called */
   type = g_inet_address_get_type ();
 
@@ -84,11 +101,37 @@ g_resolver_class_init (GResolverClass *resolver_class)
    */
   _g_resolver_addrinfo_hints.ai_socktype = SOCK_STREAM;
   _g_resolver_addrinfo_hints.ai_protocol = IPPROTO_TCP;
+
+  /**
+   * GResolver::reload:
+   * @resolver: a #GResolver
+   *
+   * Emitted when the resolver notices that the system resolver
+   * configuration has changed.
+   **/
+  signals[RELOAD] =
+    g_signal_new (I_("reload"),
+		  G_TYPE_RESOLVER,
+		  G_SIGNAL_RUN_LAST,
+		  G_STRUCT_OFFSET (GResolverClass, reload),
+		  NULL, NULL,
+		  g_cclosure_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
 }
 
 static void
 g_resolver_init (GResolver *resolver)
 {
+#ifdef G_OS_UNIX
+  struct stat st;
+#endif
+
+  resolver->priv = G_TYPE_INSTANCE_GET_PRIVATE (resolver, G_TYPE_RESOLVER, GResolverPrivate);
+
+#ifdef G_OS_UNIX
+  if (stat (_PATH_RESCONF, &st) == 0)
+    resolver->priv->resolv_conf_timestamp = st.st_mtime;
+#endif
 }
 
 static GResolver *default_resolver;
@@ -101,7 +144,7 @@ static GResolver *default_resolver;
  * many threads/processes, etc it should allocate for concurrent DNS
  * resolutions.
  *
- * Return value: the default #GResolver.
+ * Return value: (transfer full): the default #GResolver.
  *
  * Since: 2.22
  */
@@ -150,6 +193,24 @@ g_resolver_set_default (GResolver *resolver)
 }
 
 
+static void
+g_resolver_maybe_reload (GResolver *resolver)
+{
+#ifdef G_OS_UNIX
+  struct stat st;
+
+  if (stat (_PATH_RESCONF, &st) == 0)
+    {
+      if (st.st_mtime != resolver->priv->resolv_conf_timestamp)
+        {
+          resolver->priv->resolv_conf_timestamp = st.st_mtime;
+          res_init ();
+          g_signal_emit (resolver, signals[RELOAD], 0);
+        }
+    }
+#endif
+}
+
 /**
  * g_resolver_lookup_by_name:
  * @resolver: a #GResolver
@@ -178,7 +239,8 @@ g_resolver_set_default (GResolver *resolver)
  * address, it may be easier to create a #GNetworkAddress and use its
  * #GSocketConnectable interface.
  *
- * Return value: a #GList of #GInetAddress, or %NULL on error. You
+ * Return value: (element-type GInetAddress) (transfer full): a #GList
+ * of #GInetAddress, or %NULL on error. You
  * must unref each of the addresses and free the list when you are
  * done with it. (You can use g_resolver_free_addresses() to do this.)
  *
@@ -205,6 +267,7 @@ g_resolver_lookup_by_name (GResolver     *resolver,
   if (g_hostname_is_non_ascii (hostname))
     hostname = ascii_hostname = g_hostname_to_ascii (hostname);
 
+  g_resolver_maybe_reload (resolver);
   addrs = G_RESOLVER_GET_CLASS (resolver)->
     lookup_by_name (resolver, hostname, cancellable, error);
 
@@ -259,6 +322,7 @@ g_resolver_lookup_by_name_async (GResolver           *resolver,
   if (g_hostname_is_non_ascii (hostname))
     hostname = ascii_hostname = g_hostname_to_ascii (hostname);
 
+  g_resolver_maybe_reload (resolver);
   G_RESOLVER_GET_CLASS (resolver)->
     lookup_by_name_async (resolver, hostname, cancellable, callback, user_data);
 
@@ -278,8 +342,9 @@ g_resolver_lookup_by_name_async (GResolver           *resolver,
  * a value from #GResolverError. If the operation was cancelled,
  * @error will be set to %G_IO_ERROR_CANCELLED.
  *
- * Return value: a #GList of #GInetAddress, or %NULL on error. See
- * g_resolver_lookup_by_name() for more details.
+ * Return value: (element-type GInetAddress) (transfer full): a #GList
+ * of #GInetAddress, or %NULL on error. See g_resolver_lookup_by_name()
+ * for more details.
  *
  * Since: 2.22
  */
@@ -312,7 +377,7 @@ g_resolver_lookup_by_name_finish (GResolver     *resolver,
 }
 
 /**
- * g_resolver_free_addresses:
+ * g_resolver_free_addresses: (skip)
  * @addresses: a #GList of #GInetAddress
  *
  * Frees @addresses (which should be the return value from
@@ -363,6 +428,7 @@ g_resolver_lookup_by_address (GResolver     *resolver,
   g_return_val_if_fail (G_IS_RESOLVER (resolver), NULL);
   g_return_val_if_fail (G_IS_INET_ADDRESS (address), NULL);
 
+  g_resolver_maybe_reload (resolver);
   return G_RESOLVER_GET_CLASS (resolver)->
     lookup_by_address (resolver, address, cancellable, error);
 }
@@ -391,6 +457,7 @@ g_resolver_lookup_by_address_async (GResolver           *resolver,
   g_return_if_fail (G_IS_RESOLVER (resolver));
   g_return_if_fail (G_IS_INET_ADDRESS (address));
 
+  g_resolver_maybe_reload (resolver);
   G_RESOLVER_GET_CLASS (resolver)->
     lookup_by_address_async (resolver, address, cancellable, callback, user_data);
 }
@@ -480,9 +547,9 @@ g_resolver_get_service_rrname (const char *service,
  * to create a #GNetworkService and use its #GSocketConnectable
  * interface.
  *
- * Return value: a #GList of #GSrvTarget, or %NULL on error. You must
- * free each of the targets and the list when you are done with it.
- * (You can use g_resolver_free_targets() to do this.)
+ * Return value: (element-type GSrvTarget) (transfer full): a #GList of #GSrvTarget,
+ * or %NULL on error. You must free each of the targets and the list when you are
+ * done with it. (You can use g_resolver_free_targets() to do this.)
  *
  * Since: 2.22
  */
@@ -504,6 +571,7 @@ g_resolver_lookup_service (GResolver     *resolver,
 
   rrname = g_resolver_get_service_rrname (service, protocol, domain);
 
+  g_resolver_maybe_reload (resolver);
   targets = G_RESOLVER_GET_CLASS (resolver)->
     lookup_service (resolver, rrname, cancellable, error);
 
@@ -547,6 +615,7 @@ g_resolver_lookup_service_async (GResolver           *resolver,
 
   rrname = g_resolver_get_service_rrname (service, protocol, domain);
 
+  g_resolver_maybe_reload (resolver);
   G_RESOLVER_GET_CLASS (resolver)->
     lookup_service_async (resolver, rrname, cancellable, callback, user_data);
 
@@ -566,8 +635,8 @@ g_resolver_lookup_service_async (GResolver           *resolver,
  * a value from #GResolverError. If the operation was cancelled,
  * @error will be set to %G_IO_ERROR_CANCELLED.
  *
- * Return value: a #GList of #GSrvTarget, or %NULL on error. See
- * g_resolver_lookup_service() for more details.
+ * Return value: (element-type GSrvTarget) (transfer full): a #GList of #GSrvTarget,
+ * or %NULL on error. See g_resolver_lookup_service() for more details.
  *
  * Since: 2.22
  */
@@ -591,7 +660,7 @@ g_resolver_lookup_service_finish (GResolver     *resolver,
 }
 
 /**
- * g_resolver_free_targets:
+ * g_resolver_free_targets: (skip)
  * @targets: a #GList of #GSrvTarget
  *
  * Frees @targets (which should be the return value from
@@ -691,13 +760,13 @@ _g_resolver_addresses_from_addrinfo (const char       *hostname,
 void
 _g_resolver_address_to_sockaddr (GInetAddress            *address,
                                  struct sockaddr_storage *sa,
-                                 gsize                   *sa_len)
+                                 gsize                   *len)
 {
   GSocketAddress *sockaddr;
 
   sockaddr = g_inet_socket_address_new (address, 0);
   g_socket_address_to_native (sockaddr, (struct sockaddr *)sa, sizeof (*sa), NULL);
-  *sa_len = g_socket_address_get_native_size (sockaddr);
+  *len = g_socket_address_get_native_size (sockaddr);
   g_object_unref (sockaddr);
 }
 
@@ -862,6 +931,3 @@ _g_resolver_targets_from_DnsQuery (const gchar  *rrname,
 }
 
 #endif
-
-#define __G_RESOLVER_C__
-#include "gioaliasdef.c"
