@@ -29,10 +29,10 @@
  * Modified by the GLib Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GLib Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GLib at ftp://ftp.gtk.org/pub/gtk/.  
+ * GLib at ftp://ftp.gtk.org/pub/gtk/.
  */
 
-/* 
+/*
  * MT safe
  */
 
@@ -47,7 +47,12 @@
 #include <unistd.h>
 #endif
 
-#include "glib.h"
+#include "grand.h"
+
+#include "gmain.h"
+#include "gmem.h"
+#include "gtestutils.h"
+#include "gthread.h"
 #include "gthreadprivate.h"
 
 #ifdef G_OS_WIN32
